# SPDX-FileCopyrightText: 2025 Florian Obersteiner <f.obersteiner@kit.edu>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import logging as log
from datetime import datetime
from pathlib import Path
from typing import Iterable

import pandas as pd
import xarray as xr
from siphon.catalog import Dataset as SiphonDataset

from .config import DotDict


def load_from_thredds(
    datasets: list[SiphonDataset], config: DotDict, apconfig: pd.DataFrame, drop: bool = False
) -> (pd.DataFrame, dict[int, str]):  # type: ignore
    """
    Load multiple netCDF files from a THREDS TDS into a single DataFrame.
    """
    print("begin load data from THREDDS...")
    dataframes = []
    airports = {}

    for i, ds in enumerate(datasets):
        fullset = datasets[i].remote_access(use_xarray=True).to_dataframe()
        if drop:
            df = pd.DataFrame()
            for v in config.VARIABLES:
                df[v] = fullset[v]
        else:
            df = fullset

        # extract flight number and date from filename; add as extra columns
        df["flight_number"] = int(ds.name.split("_")[2])
        airports[int(ds.name.split("_")[2])] = (
            f"from: {ds.name.split('_')[3]} ({apconfig.loc[ds.name.split('_')[3], :].Name}), to: {ds.name.split('_')[4]} ({apconfig.loc[ds.name.split('_')[4], :].Name})"
        )

        df["date"] = datetime.strptime(ds.name.split("_")[1], "%Y%m%d").strftime("%Y-%m-%d")
        # make a linear interpolation of the acetonitrile column for BB flaggin
        if "ACN" in df.columns:
            if df["ACN"].isna().all():
                log.warning("'ACN' column is all NaN values!")
            df["BB_flag"] = df["ACN"].interpolate() > (
                config.BB_THRESH + config.ACN_PRC_NSIGMA * df["ACN_prc"].interpolate()
            )
        else:
            raise ValueError(f"ACN column not found in {ds.name}")

        dataframes.append(df)

    print("Data loaded from THREDDS successfully")
    return (pd.concat(dataframes), airports)


def load_from_disk(
    paths: Iterable[Path], config: DotDict, apconfig: pd.DataFrame, drop: bool = False
) -> (pd.DataFrame, dict[int, str]):  # type: ignore
    """
    Load multiple netCDF files into a single DataFrame.
    """
    dataframes = []
    airports = {}

    for path in sorted(paths):
        fullset = xr.open_dataset(path).to_dataframe()
        if drop:
            df = pd.DataFrame()
            for v in config.VARIABLES:
                df[v] = fullset[v]
        else:
            df = fullset

        # extract flight number and date from filename; add as extra columns
        df["flight_number"] = int(path.name.split("_")[2])
        airports[int(path.name.split("_")[2])] = (
            f"from: {path.name.split('_')[3]} ({apconfig.loc[path.name.split('_')[3], :].Name}), to: {path.name.split('_')[4]} ({apconfig.loc[path.name.split('_')[4], :].Name})"
        )

        df["date"] = datetime.strptime(path.stem.split("_")[1], "%Y%m%d").strftime("%Y-%m-%d")
        # make a linear interpolation of the acetonitrile column for BB flaggin
        if "ACN" in df.columns:
            if df["ACN"].isna().all():
                log.warning("'ACN' column is all NaN values!")
            df["BB_flag"] = df["ACN"].interpolate() > (
                config.BB_THRESH + config.ACN_PRC_NSIGMA * df["ACN_prc"].interpolate()
            )
        else:
            raise ValueError(f"ACN column not found in {path}")

        dataframes.append(df)

    log.info("Data loaded from disk successfully")
    return (pd.concat(dataframes), airports)
